/*
 * Asterisk -- A telephony toolkit for Linux.
 *
 * popen application
 *
 * Copyright (C) 2005 Matt O'Gorman.  All rights reserved.
 * popen info - Runs a command and returns what command returns.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <asterisk/file.h>
#include <asterisk/logger.h>
#include <asterisk/options.h>
#include <asterisk/channel.h>
#include <asterisk/pbx.h>
#include <asterisk/module.h>

static char *tdesc = "Stores output of command into a variable";

static char *app_popen = "Popen";

static char *popen_synopsis = "Popen(Program|Varname|Length)";

static char *popen_descrip =
"Popen(program,varname,length)\n"
"  Program - The name of the program to run, if c flag not called\n"
"  Varname   - Variable you want to copy to\n"
"  Length - Max length of variable.  if not set is infinite\n"
;

STANDARD_LOCAL_USER;

LOCAL_USER_DECL;

char * ast_popen(int len, char *prog)
{
    FILE *ast_command;
    char line[160];
    int i=1;
    int flag=0;
    char *String=NULL;
    String=(char *)malloc(1*sizeof(char));
    ast_command  = popen(prog, "r");
    
    while(fgets(line,sizeof line,ast_command)&&(i < len||len==0))
    {
		if(flag==0)
		{
			String=(char *)malloc((strlen(line)+1)*sizeof(char));
			strcpy(String,line);
			flag=1;
		}
		else
		{ 
        	if (String)
        	{ 
        	    String = (char *) realloc(String, (strlen(String)+strlen(line)+1)*sizeof(char));
        	    strcat(String,line);
        	    i++;
        	}
        	else
				ast_log(LOG_ERROR, "Out of memory\n");
    	}
    }
	
    pclose(ast_command);
    return String;
}

static int popen_exec(struct ast_channel *chan, void *data)
{
	int res=0;
	struct localuser *u;
	char *s, *varname=NULL, *program=NULL, *length=NULL, *returnvar=NULL;
	int args_okay = 0;
	int len=0;
	
	LOCAL_USER_ADD(u);
	
	/* check my data */
	if (data)
	{
		s = ast_strdupa((char *)data);
		if (s)
		{
            program = strsep(&s, "|");
            if (program && (program[0] != '\0'))
            {
                varname = strsep(&s, "|");
				if (varname)
				{
				    length = strsep(&s, "|");
				    if (length)
				    {
						if(!sscanf(length,"%d",&len))
            				ast_log(LOG_ERROR, "Length is not a number, set length to max");
						args_okay = 1;
					}	
					else
					{
						len = 0;
						args_okay=1;
				    }
				}
            }
        }
        
        else
        {
			ast_log(LOG_ERROR, "Out of memory\n");
			res = -1;
		}
	}
	
    /*good data */
    
	if (args_okay)
	{
        returnvar=ast_popen(len,program);
        pbx_builtin_setvar_helper(chan, varname, returnvar);
		
	}
	
	LOCAL_USER_REMOVE(u);
	return res;
}


int unload_module(void)
{
	STANDARD_HANGUP_LOCALUSERS;
	return ast_unregister_application(app_popen);
}

int load_module(void)
{
	return ast_register_application(app_popen, popen_exec, popen_synopsis, popen_descrip);
}

char *description(void)
{
	return tdesc;
}

int usecount(void)
{
	int res;
	STANDARD_USECOUNT(res);
	return res;
}

char *key()
{
	return ASTERISK_GPL_KEY;
}
